//
//  EAGLView.m
//  Weeek3
//
//  Created by Ay on 2008/9/17.
//  Copyright NA 2008. All rights reserved.
//



#import <QuartzCore/QuartzCore.h>
#import <OpenGLES/EAGLDrawable.h>

#define USE_DEPTH_BUFFER 0
#define DEGREES_TO_RADIANS(__ANGLE__) ((__ANGLE__) / 180.0 * M_PI)

#define FIRST_LEVEL -2.0
#define SECOND_LEVEL -3.0
#define SHOW_CHANGE 0
#define SHOW_RANGE 5
#define SCORES_PER_EACH 20
#define SCORES_DEDUCE 5

#import "EAGLView.h"

// A class extension to declare private methods
@interface EAGLView ()

@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) NSTimer *animationTimer;

- (BOOL) createFramebuffer;
- (void) destroyFramebuffer;

@end


@implementation EAGLView
@synthesize context;
@synthesize animationTimer;
@synthesize animationInterval;
@synthesize scores;

// You must implement this
+ (Class)layerClass {
	return [CAEAGLLayer class];
}


//The GL view is stored in the nib file. When it's unarchived it's sent -initWithCoder:
- (id)initWithCoder:(NSCoder*)coder {

	if ((self = [super initWithCoder:coder])) {
		// Get the layer
		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
		
		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
		   [NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
		
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
		
		if (!context || ![EAGLContext setCurrentContext:context] || ![self createFramebuffer]) {
			[self release];
			return nil;
		}
		NSLog(@"initWithCode");
		animationInterval = 1.0 / 60.0;
		scores = 0;
		
		bg =[ [Texture2D alloc] initWithImagePath:@"lime-green-bubbles-iphone-wallpaper.jpg"];
		
		[self setupView];
		//[self drawView];
	}
	return self;
}



- (void)layoutSubviews {
	[EAGLContext setCurrentContext:context];
	[self destroyFramebuffer];
	[self createFramebuffer];
	[self drawView];
}


- (BOOL)createFramebuffer {
	
	glGenFramebuffersOES(1, &viewFramebuffer);
	glGenRenderbuffersOES(1, &viewRenderbuffer);
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
	glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, viewRenderbuffer);
	
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
	if (USE_DEPTH_BUFFER) {
		glGenRenderbuffersOES(1, &depthRenderbuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);
		glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);
	}

	if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
		NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
		return NO;
	}
	
	return YES;
}


- (void)destroyFramebuffer {
	
	glDeleteFramebuffersOES(1, &viewFramebuffer);
	viewFramebuffer = 0;
	glDeleteRenderbuffersOES(1, &viewRenderbuffer);
	viewRenderbuffer = 0;
	
	if(depthRenderbuffer) {
		glDeleteRenderbuffersOES(1, &depthRenderbuffer);
		depthRenderbuffer = 0;
	}
}


- (void)startAnimation {
	animationTimer = [NSTimer scheduledTimerWithTimeInterval:animationInterval target:self selector:@selector(drawView) userInfo:nil repeats:YES];
}


- (void)stopAnimation {
	animationTimer = nil;
}


- (void)setAnimationTimer:(NSTimer *)newTimer {
	[animationTimer invalidate];
	animationTimer = newTimer;
}


- (void)setAnimationInterval:(NSTimeInterval)interval {
	
	animationInterval = interval;
	if (animationTimer) {
		[self stopAnimation];
		[self startAnimation];
	}
}

// Sets up an array of values to use as the sprite vertices.
const GLfloat spriteVertices[] = {
-0.5f, -0.5f,
0.5f, -0.5f,
-0.5f,  0.5f,
0.5f,  0.5f,
};

// Sets up an array of values for the texture coordinates.
const GLshort spriteTexcoords[] = {
1, 1,
0, 1,
1, 0,
0, 0,
};




const GLfloat spriteLocation[] ={
-0.65,0.4,
0.0,0.4,
0.65,0.4,

-0.65,-0.3,
0.0,-0.3,
0.65,-0.3,

-0.65,-1.0,
0.0,-1.0,
0.65,-1.0,

};

GLfloat spriteEventCount[] ={
0,0,0,0,0,0,0,0,0
};

GLfloat spriteEventDir[] ={
1,-1,1,-1,1,-1,1,-1,1
};

- (void)setupView
{
	CGImageRef spriteImage,bgImage;
	CGContextRef spriteContext,bgContext;
	GLubyte *spriteData,*bgData;
	size_t	width, height, bgWidth,bgHeight;
	
	
	// Sets up matrices and transforms for OpenGL ES
	glViewport(0, 0, backingWidth, backingHeight);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//glOrthof(-1.0f, 1.0f, -1.5f, 1.5f, -1.0f, 1.0f);
	[self setViewFrustum];
	glMatrixMode(GL_MODELVIEW);
	// Clears the view with black
	glClearColor(0.5f, 0.7f, 0.8f, 1.0f);
	
	// Sets up pointers and enables states needed for using vertex arrays and textures
	{
		//select sprite
		glVertexPointer(2, GL_FLOAT, 0, spriteVertices);
		glEnableClientState(GL_VERTEX_ARRAY);
		glTexCoordPointer(2, GL_SHORT, 0, spriteTexcoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	}
	// Creates a Core Graphics image from an image file
	spriteImage = [UIImage imageNamed:@"00-2.png"].CGImage;
	bgImage = [UIImage imageNamed:@"00-1.png"].CGImage;
	
	
	// Get the width and height of the image
	width = CGImageGetWidth(spriteImage);
	height = CGImageGetHeight(spriteImage);
	
	bgWidth  = CGImageGetWidth(bgImage);
	bgHeight  = CGImageGetWidth(bgImage);
	// Texture dimensions must be a power of 2. If you write an application that allows users to supply an image,
	// you'll want to add code that checks the dimensions and takes appropriate action if they are not a power of 2.
	
	if(spriteImage && bgImage) {
		
		// Allocated memory needed for the bitmap context
		spriteData = (GLubyte *) malloc(width * height * 4);
		bgData = (GLubyte *) malloc(bgWidth * bgHeight * 4);
		
		// Uses the bitmatp creation function provided by the Core Graphics framework. 
		spriteContext = CGBitmapContextCreate(spriteData, width, height, 8, width * 4, CGImageGetColorSpace(spriteImage), kCGImageAlphaPremultipliedLast);
		bgContext =  CGBitmapContextCreate(bgData, bgWidth, bgHeight, 8, bgWidth * 4, CGImageGetColorSpace(bgImage), kCGImageAlphaPremultipliedLast);
		
		// After you create the context, you can draw the sprite image to the context.
		CGContextDrawImage(spriteContext, CGRectMake(0.0, 0.0, (CGFloat)width, (CGFloat)height), spriteImage);
		CGContextDrawImage(bgContext, CGRectMake(0.0, 0.0, (CGFloat)bgWidth, (CGFloat)bgHeight), bgImage);
		
		// You don't need the context at this point, so you need to release it to avoid memory leaks.
		CGContextRelease(spriteContext);
		CGContextRelease(bgContext);
		
		// Use OpenGL ES to generate a name for the texture.
		glGenTextures(3, spriteTexture);
		{
		// Bind the texture name. 
		glBindTexture(GL_TEXTURE_2D, spriteTexture[0]);
		// Speidfy a 2D texture image, provideing the a pointer to the image data in memory
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, spriteData);
		// Release the image data
		free(spriteData);
		// Set the texture parameters to use a minifying filter and a linear filer (weighted average)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		}
		{
		glBindTexture(GL_TEXTURE_2D, spriteTexture[1]);
		// Speidfy a 2D texture image, provideing the a pointer to the image data in memory
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bgWidth, bgHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, bgData);
		// Release the image data
		free(bgData);
		// Set the texture parameters to use a minifying filter and a linear filer (weighted average)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		}
	}
}

- (void) setViewFrustum{
	const GLfloat	zNear = 0.1,
					zFar = 1000.0,
					fieldOfView = 60.0;
	GLfloat			size;
	
	//Set the OpenGL projection matrix
	glMatrixMode(GL_PROJECTION);
	size = zNear * tanf(DEGREES_TO_RADIANS(fieldOfView) / 2.0);
	CGRect rect = self.bounds;
	glFrustumf(-size, size, -size / (rect.size.width / rect.size.height), size / (rect.size.width / rect.size.height), zNear, zFar);
	glViewport(0, 0, rect.size.width, rect.size.height);
		
}

- (void)drawView {
	if(gameCount++ == 180){
		gameCount=0;
	}
	
	// Make sure that you are drawing to the current context
	[EAGLContext setCurrentContext:context];
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);

	//glRotatef(3.0f, 0.0f, 0.0f, 1.0f);
	//glTranslatef(0.01, 0.0,-0.01);

	glClear(GL_COLOR_BUFFER_BIT);
	
	//[bg drawAtPoint:CGPointMake(0,420)];
	if(gameCount==0){
		//NSLog(@"Render");
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(showStatus[0][i][j]==2)
					continue;
				if(rand()%SHOW_RANGE ==SHOW_CHANGE){
					showStatus[0][i][j] = 1;
				}else{ 
					showStatus[0][i][j] = 0;
				}
			}
		}
	}
	
	for(int i=0;i<9;i++){
		int x = i*2;
		int y =i*2+1;
		if(showStatus[0][i/3][i%3]==1){
			
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, spriteTexture[0]);
			// Set a blending function to use
			glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
			// Enable blending
			glEnable(GL_BLEND);

			glPushMatrix();
			glTranslatef(spriteLocation[x],spriteLocation[y],FIRST_LEVEL);
			//glRotatef(45.0, 0, 1.0, 0);
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glPopMatrix();

		}else if(showStatus[0][i/3][i%3]==2){
			spriteEventCount[i] += 0.05;
			if(spriteEventCount[i]>3){
				showStatus[0][i/3][i%3]=0;
				spriteEventCount[i]=0;
				if(rand()%2){
					spriteEventDir[i] = 1;
				}else{
					spriteEventDir[i] = -1;
				}
				continue;
			}
			glPushMatrix();
			glTranslatef(spriteLocation[x],spriteLocation[y],FIRST_LEVEL-spriteEventDir[i]*spriteEventCount[i]);
			glRotatef(spriteEventCount[i] *60.0, 0, 0.0, 1);
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glPopMatrix();
		}else{
			/*
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, spriteTexture[1]);
			// Set a blending function to use
			glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
			// Enable blending
			glEnable(GL_BLEND);
			
			glPushMatrix();
			glTranslatef(spriteLocation[x],spriteLocation[y],FIRST_LEVEL);
			//glRotatef(45.0, 0, 1.0, 0);
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glPopMatrix();
			 */
		}
	}
	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context presentRenderbuffer:GL_RENDERBUFFER_OES];
	
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {	
	NSLog( @"Touches Began");
}



- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	NSLog(@"Touches Moved");
	CGPoint point = [[touches anyObject] locationInView:self];
	
	NSLog([NSString stringWithFormat:@"%0.2f", point.x]);
	NSLog([NSString stringWithFormat:@"%0.2f", point.y]);
	
	for(int i =0;i<1;i++){
		int y = lrint(point.x - 60)/80;
		int x= lrint(point.y - 130)/100;
		NSLog([NSString stringWithFormat:@"showStatus %d",showStatus[0][x][y]]);
		
		if(showStatus[0][x][y] == 1){
			showStatus[0][x][y] = 2;
			scores += SCORES_PER_EACH;
			//label.text = [NSString stringWithFormat:@"scores end touch: %d",scores];
			NSLog([NSString stringWithFormat:@"scores end touch: %d",scores]);
		}
	}
	
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	NSLog(@"No Touch Event");
	
	for (UITouch *touch in touches) {
		CGPoint point = [touch locationInView:self];
		
		NSLog([NSString stringWithFormat:@"%0.2f", point.x]);
		NSLog([NSString stringWithFormat:@"%0.2f", point.y]);
		
		for(int i =0;i<1;i++){
			int y = lrint(point.x - 60)/80;
			int x= lrint(point.y - 130)/100;
			NSLog([NSString stringWithFormat:@"showStatus %d",showStatus[0][x][y]]);
			
			if(showStatus[0][x][y] == 1){
				showStatus[0][x][y] = 2;
				scores += SCORES_PER_EACH;
				//label.text = [NSString stringWithFormat:@"scores end touch: %d",scores];
				NSLog([NSString stringWithFormat:@"scores end touch: %d",scores]);
			}
		}
		
		if (touch.tapCount == 2) {
			NSLog(@"Double Tapped");
		}else if(touch.tapCount>2){
			NSLog([NSString stringWithFormat:@"Tap Count : %d",touch.tapCount]);
		}
	}
}


- (void)dealloc {
	
	[self stopAnimation];
	
	if ([EAGLContext currentContext] == context) {
		[EAGLContext setCurrentContext:nil];
	}
	
	[context release];	
	[super dealloc];
}

@end
